//*******************************************************
//
//	VS1011 + MMC/SD MP3 Player BK1
//
//	MCU:ATmega168
//		efuse:0xF9
//		hfuse:0xD6	BOD 1.8V EEPROMی
//		lfuse:0xE2	CRU 8MHz
//
//	Compiler:	AVR-GCC		3.4.6
//				AVR-LIBC	1.4.2
//				binutils	2.16.1
//
//	2007.3.24	KAWAKAMI Yukio
//*******************************************************

#define		CR	13
#define		LF	10
#define		ON	1
#define		OFF	0
#define		TRUE	1
#define		FALSE	0

#define		FAT12		1
#define		FAT16		2

#define		BEEP_HIGH	0
#define		BEEP_LOW	1

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;
typedef unsigned short	ushort;

struct FDATE {
	unsigned short	ft_day	 : 5;
	unsigned short	ft_month : 4;
	unsigned short	ft_year	 : 7;
};

struct FTIME {
	unsigned short	ft_sec	: 5;
	unsigned short	ft_min	: 6;
	unsigned short	ft_hour	: 5;
};

struct DIRECTORY {
	uchar	filename[8];
	uchar	ext[3];
	uchar	att;
	uchar	not_use[10];
	struct FTIME	time;
	struct FDATE	date;
	unsigned short	cluster;
	ulong	size;
};

struct FILE_DATA {
	ulong	sec;
	uint	sc;
	uint	cluster;
	ushort	size;
	ushort	p;
};


#define		MP3_DATA_CS	0	// PORTB
#define		MP3_CTRL_CS	1

#define		MP3_RESET	2	// PORTC
#define		MP3_DREQ	1

#define		LED			3	// PORTD

#define		POWER		3	// PORTC
#define		BATT_VOLT	0	// PORTC drd

#define		RXD			0	// PORTD
#define		TXD			1
#define		ROM_CS		2

#define		SCL			5	// PORTC
#define		SDA			4

#define		PB7			7	// PORTB
#define		PB6			6

// MMC CSM蓖
#define		MMC_MISO	4	// PORTB
#define		MMC_MOSI	3
#define		MMC_SCK		5

#define		MMC_CS_PORT	PORTB
#define		MMC_CS_DDR	DDRB
#define		MMC_CS		2

#define		MMC_CS_ON	MMC_CS_PORT &= ~(1<<MMC_CS)
#define		MMC_CS_OFF	MMC_CS_PORT |= (1<<MMC_CS)

// XCb`tO̘_rbg
#define		SW_START	7
#define		SW_STOP		7
#define		SW_FF		7	// START/STOP/FF p
#define		SW_REW		6
#define		SW_VOLDOWN	5
#define		SW_VOLUP	4

// XCb`̕rbg
#define		SW_PORT_START	4	// PIND
#define		SW_PORT_STOP	4	// == SW_PORT_START
#define		SW_PORT_REW		5	// PIND
#define		SW_PORT_VOLUP	6	// PIND
#define		SW_PORT_VOLDOWN	7	// PIND

// EEPROM AhX
#define		EEPROM_VOLUME		5	// {[
#define		EEPROM_MUSIC		6	// Đ̋Ȕԍ
#define		EEPROM_CLUSTER		8	// Đ̃NX^
#define		EEPROM_REMAIN		10	// Đ̋Ȃ̎cZN^
#define		EEPROM_SIZE			14	// Đ̋ȃf[^̒

#define		EEPROM_LOW_BATT		18	// [obe[xl
#define		EEPROM_SHUTDOWN		20	// [obe[Vbg_El
#define		EEPROM_VS_CLOCK		22	// VS1011eNbN
#define		EEPROM_SLEEP		26	// X[v^C}()

// VS1011e WX^Erbg
#define	SCI_MODE		0
#define	SM_RESET		(1<<2)
#define	SM_TESTS		(1<<5)
#define	SM_SDINEW		(1<<11)

#define	SCI_STATUS		1
#define	SS_APDOWN2		(1<<3)
#define	SS_APDOWN1		(1<<2)

#define	SCI_BASS		2
#define	SCI_CLOCKF		3
#define	SCI_DECODE_TIME	4
#define	SCI_AUDATA		5
#define	SCI_HDAT0		8
#define	SCI_HDAT1		9
#define	SCI_VOL			11


uchar	InitCard(void);
uchar	SPI_read_open(ulong adrs);
void	SPI_read_close(void);
uchar	SPI_command(uchar com, ulong arg);
uchar	SPI_in(void);
void	SPI_out(uchar data);
void	SPI_dummy_clock(void);
void	SPI_wait(void);
void	InitWait(void);

uint	ADCin(char ch);

void	read_32(ulong addr, uchar *buff);
uchar	read_VFAT_info(void);
int		read_card(ulong addr);

uint	search_MP3(uint num, struct DIRECTORY *dir);
char	play_music(uint n, uint clst, long remainbyte);
uint	check_cont_cluster(uint c);
uint	read_word(ulong addr);

uint	next_cluster(uint c);
ulong	cluster_to_adrs(uint cluster);
void	open_file(struct DIRECTORY *dir, struct FILE_DATA *fd);
void	next_sector(struct FILE_DATA *fd);

void	beep(uchar n, char err);
void	beep_wait(uchar n);

void	MP3_sin(PGM_VOID_P data);
void	MP3_init(void);
void	MP3_command(uchar addr, uint arg);
void	MP3_com_write(uchar data);
uint	MP3_command_read(uchar addr);
void	MP3_wait(void);
void	wait_10ms(uchar n);
void	VS1011_reset(void);
void	set_volume(uchar vol);

void	shutdown(char flag);
int		check_batt(void);
void	switch_wait(void);

void	write_eeprom(uchar adrs, uchar data);
uchar	read_eeprom(uchar adrs);
void	write_eeprom_word(uint adrs, ushort data);
ushort	read_eeprom_word(uint adrs);
void	write_eeprom_long(uint adrs, ulong data);
ulong	read_eeprom_long(uint adrs);

